@@ -103,7 +103,7 @@ static void do_ping(struct cgpu_info *modminer)
 		modminer->drv->name, modminer->fpgaid, amount, err);
 
 	// Clear any outstanding data
-	while ((err = usb_read(modminer, buf, sizeof(buf)-1, &amount, C_CLEAR)) == 0 && amount > 0)
+	while ((err = usb_read_once(modminer, buf, sizeof(buf)-1, &amount, C_CLEAR)) == 0 && amount > 0)
 		applog(LOG_DEBUG, "%s%u: clear got %d",
 			modminer->drv->name, modminer->fpgaid, amount);
 
@@ -126,13 +126,8 @@ static bool modminer_detect_one(struct libusb_device *dev, struct usb_find_devic
 	mutex_init(modminer->modminer_mutex);
 	modminer->fpgaid = (char)0;
 
-	if (!usb_init(modminer, dev, found)) {
-		applog(LOG_ERR, "%s detect (%d:%d) failed to initialise (incorrect device?)",
-			modminer->drv->dname,
-			(int)(modminer->usbinfo.bus_number),
-			(int)(modminer->usbinfo.device_address));
+	if (!usb_init(modminer, dev, found))
 		goto shin;
-	}
 
 	sprintf(devpath, "%d:%d",
 			(int)(modminer->usbinfo.bus_number),
@@ -146,7 +141,7 @@ static bool modminer_detect_one(struct libusb_device *dev, struct usb_find_devic
 		goto unshin;
 	}
 
-	if ((err = usb_read(modminer, buf, sizeof(buf)-1, &amount, C_GETVERSION)) < 0 || amount < 1) {
+	if ((err = usb_read_once(modminer, buf, sizeof(buf)-1, &amount, C_GETVERSION)) < 0 || amount < 1) {
 		if (err < 0)
 			applog(LOG_ERR, "%s detect (%s) no version reply (%d)",
 				modminer->drv->dname, devpath, err);
@@ -303,7 +298,7 @@ static bool get_info(struct cgpu_info *modminer, FILE *f, char *buf, int bufsiz,
 	}
 
 	if (fread(buf, len, 1, f) != 1) {
-		applog(LOG_ERR, "%s%u: Error (%d) reading bitstream '%s'", errno,
+		applog(LOG_ERR, "%s%u: Error (%d) reading bitstream '%s'",
 			modminer->drv->name, modminer->device_id, errno, name);
 		return false;
 	}
@@ -441,7 +436,7 @@ static bool modminer_fpga_upload_bitstream(struct cgpu_info *modminer)
 		goto dame;
 	}
 
-	applog(LOG_DEBUG, " Version: %u, build %u", (fwusercode >> 8) & 0xff, fwusercode & 0xff);
+	applog(LOG_DEBUG, " Version: %lu, build %lu", (fwusercode >> 8) & 0xff, fwusercode & 0xff);
 
 	if (!get_expect(modminer, f, 'b'))
 		goto undame;
@@ -519,7 +514,7 @@ static bool modminer_fpga_upload_bitstream(struct cgpu_info *modminer)
 		if (fread(buf, buflen, 1, f) != 1) {
 			mutex_unlock(modminer->modminer_mutex);
 
-			applog(LOG_ERR, "%s%u: bitstream file read error %d (%d bytes left)",
+			applog(LOG_ERR, "%s%u: bitstream file read error %d (%lu bytes left)",
 				modminer->drv->name, modminer->device_id, errno, len);
 
 			goto dame;
@@ -534,7 +529,7 @@ static bool modminer_fpga_upload_bitstream(struct cgpu_info *modminer)
 				ptr += amount;
 
 				if (opt_debug)
-					applog(LOG_DEBUG, "%s%u: Program timeout (%d:%d) sent %d tries %d",
+					applog(LOG_DEBUG, "%s%u: Program timeout (%d:%d) sent %zu tries %d",
 						modminer->drv->name, modminer->device_id,
 						amount, err, remaining, tries);
 
@@ -544,7 +539,7 @@ static bool modminer_fpga_upload_bitstream(struct cgpu_info *modminer)
 			} else {
 				mutex_unlock(modminer->modminer_mutex);
 
-				applog(LOG_ERR, "%s%u: Program failed (%d:%d) sent %d",
+				applog(LOG_ERR, "%s%u: Program failed (%d:%d) sent %zu",
 					modminer->drv->name, modminer->device_id, amount, err, remaining);
 
 				goto dame;
@@ -559,7 +554,7 @@ static bool modminer_fpga_upload_bitstream(struct cgpu_info *modminer)
 		upto = (float)(totlen - len) / (float)(totlen);
 		if (upto >= nextmsg) {
 			applog(LOG_WARNING,
-				"%s%u: Programming %.1f%% (%d out of %d)",
+				"%s%u: Programming %.1f%% (%lu out of %lu)",
 				modminer->drv->name, modminer->device_id, upto*100, (totlen - len), totlen);
 
 			nextmsg += 0.1;
@@ -590,7 +585,7 @@ static bool modminer_fpga_prepare(struct thr_info *thr)
 	struct cgpu_info *modminer = thr->cgpu;
 	struct timeval now;
 
-	gettimeofday(&now, NULL);
+	cgtime(&now);
 	get_datestamp(modminer->init, &now);
 
 	struct modminer_fpga_state *state;
@@ -785,10 +780,10 @@ static bool modminer_start_work(struct thr_info *thr, struct work *work)
 	memcpy(&cmd[34], work->data + 64, 12);
 
 	if (state->first_work.tv_sec == 0)
-		gettimeofday(&state->first_work, NULL);
+		cgtime(&state->first_work);
 
 	if (state->last_nonce.tv_sec == 0)
-		gettimeofday(&state->last_nonce, NULL);
+		cgtime(&state->last_nonce);
 
 	mutex_lock(modminer->modminer_mutex);
 
@@ -801,7 +796,7 @@ static bool modminer_start_work(struct thr_info *thr, struct work *work)
 		return false;
 	}
 
-	gettimeofday(&state->tv_workstart, NULL);
+	cgtime(&state->tv_workstart);
 
 	sta = get_status(modminer, "start work", C_SENDWORKSTATUS);
 
@@ -971,7 +966,7 @@ static uint64_t modminer_process_results(struct thr_info *thr, struct work *work
 			curr_hw_errors = state->hw_errors;
 			submit_nonce(thr, work, nonce);
 			if (state->hw_errors > curr_hw_errors) {
-				gettimeofday(&now, NULL);
+				cgtime(&now);
 				// Ignore initial errors that often happen
 				if (tdiff(&now, &state->first_work) < 2.0) {
 					state->shares = 0;
@@ -986,7 +981,7 @@ static uint64_t modminer_process_results(struct thr_info *thr, struct work *work
 					}
 				}
 			} else {
-				gettimeofday(&state->last_nonce, NULL);
+				cgtime(&state->last_nonce);
 				state->death_stage_one = false;
 				// If we've reached the required good shares in a row then clock up
 				if (((state->shares - state->shares_last_hw) >= state->shares_to_good) &&
@@ -996,7 +991,7 @@ static uint64_t modminer_process_results(struct thr_info *thr, struct work *work
 		} else {
 			// on rare occasions - the MMQ can just stop returning valid nonces
 			double death = ITS_DEAD_JIM * (state->death_stage_one ? 2.0 : 1.0);
-			gettimeofday(&now, NULL);
+			cgtime(&now);
 			if (tdiff(&now, &state->last_nonce) >= death) {
 				if (state->death_stage_one) {
 					modminer_delta_clock(thr, MODMINER_CLOCK_DEAD, false, true);
@@ -1037,7 +1032,7 @@ tryagain:
 		else
 			timeout = processtime;
 
-		gettimeofday(&now, NULL);
+		cgtime(&now);
 		if (tdiff(&now, &state->tv_workstart) > timeout)
 			break;
 
@@ -1048,7 +1043,7 @@ tryagain:
 	}
 
 	struct timeval tv_workend, elapsed;
-	gettimeofday(&tv_workend, NULL);
+	cgtime(&tv_workend);
 	timersub(&tv_workend, &state->tv_workstart, &elapsed);
 
 	// Not exact since the clock may have changed ... but close enough I guess
@@ -1074,7 +1069,7 @@ static int64_t modminer_scanhash(struct thr_info *thr, struct work *work, int64_
 
 	// Don't start new work if overheated
 	if (state->overheated == true) {
-		gettimeofday(&tv1, NULL);
+		cgtime(&tv1);
 
 		while (state->overheated == true) {
 			check_temperature(thr);
@@ -1084,7 +1079,7 @@ static int64_t modminer_scanhash(struct thr_info *thr, struct work *work, int64_
 				return -1;
 
 			if (state->overheated == true) {
-				gettimeofday(&tv2, NULL);
+				cgtime(&tv2);
 
 				// give up on this work item after 30s
 				if (work_restart(thr) || tdiff(&tv2, &tv1) > 30)
