@@ -28,7 +28,14 @@ extern int opt_log_level;
 
 /* low-level logging functions with priority parameter */
 extern void vapplog(int prio, const char *fmt, va_list ap);
-extern void applog(int prio, const char *fmt, ...);
+extern void _applog(int prio, const char *fmt, ...);
+#define applog(prio, fmt, ...) do { \
+ char *tmp42; \
+ if (0) \
+	sprintf(tmp42, fmt, ##__VA_ARGS__); \
+ else \
+	_applog(prio, fmt, ##__VA_ARGS__); \
+} while (0)
 
 /* high-level logging functions with implicit priority */
 extern void log_error(const char *fmt, ...);
