@@ -13,6 +13,15 @@
 #include "uthash.h"
 #include "logging.h"
 #include "util.h"
+#include <sys/types.h>
+#ifndef WIN32
+# include <sys/socket.h>
+# include <netdb.h>
+#endif
+
+#ifdef USE_USBUTILS
+#include <semaphore.h>
+#endif
 
 #ifdef HAVE_OPENCL
 #ifdef __APPLE_CC__
@@ -118,10 +127,13 @@ static inline int fsync (int fd)
   #include "usbutils.h"
 #endif
 
-#if !defined(WIN32) && ((__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
-#define bswap_16 __builtin_bswap16
-#define bswap_32 __builtin_bswap32
-#define bswap_64 __builtin_bswap64
+#if (!defined(WIN32) && ((__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))) \
+    || (defined(WIN32) && ((__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7)))
+#ifndef bswap_16
+ #define bswap_16 __builtin_bswap16
+ #define bswap_32 __builtin_bswap32
+ #define bswap_64 __builtin_bswap64
+#endif
 #else
 #if HAVE_BYTESWAP_H
 #include <byteswap.h>
@@ -153,6 +165,7 @@ static inline int fsync (int fd)
 # if __BYTE_ORDER == __LITTLE_ENDIAN
 #  define htole16(x) (x)
 #  define htole32(x) (x)
+#  define le32toh(x) (x)
 #  define be32toh(x) bswap_32(x)
 #  define be64toh(x) bswap_64(x)
 #  define htobe32(x) bswap_32(x)
@@ -160,6 +173,7 @@ static inline int fsync (int fd)
 # elif __BYTE_ORDER == __BIG_ENDIAN
 #  define htole16(x) bswap_16(x)
 #  define htole32(x) bswap_32(x)
+#  define le32toh(x) bswap_32(x)
 #  define be32toh(x) (x)
 #  define be64toh(x) (x)
 #  define htobe32(x) (x)
@@ -205,8 +219,8 @@ enum drv_driver {
 	DRIVER_BITFORCE,
 	DRIVER_MODMINER,
 	DRIVER_ZTEX,
-	DRIVER_CPU,
 	DRIVER_BFLSC,
+	DRIVER_AVALON,
 	DRIVER_MAX
 };
 
@@ -326,6 +340,7 @@ struct device_drv {
 
 	/* Highest target diff the device supports */
 	double max_diff;
+	double working_diff;
 };
 
 extern struct device_drv *copy_drv(struct device_drv*);
@@ -411,18 +426,21 @@ struct cgpu_info {
 	int device_id;
 	char *name;
 	char *device_path;
-	FILE *device_file;
+	void *device_data;
 	union {
 #ifdef USE_ZTEX
 		struct libztex_device *device_ztex;
 #endif
 #ifdef USE_USBUTILS
 		struct cg_usb_device *usbdev;
-#endif
-#ifdef USE_ICARUS
-		int device_fd;
 #endif
 	};
+#ifdef USE_AVALON
+	struct work **works;
+	int work_array;
+	int queued;
+	int results;
+#endif
 #ifdef USE_USBUTILS
 	struct cg_usb_info usbinfo;
 #endif
@@ -441,8 +459,10 @@ struct cgpu_info {
 	bool nonce_range;
 	bool polling;
 	bool flash_led;
+#endif /* USE_BITFORCE */
+#if defined(USE_BITFORCE) || defined(USE_BFLSC)
 	pthread_mutex_t device_mutex;
-#endif
+#endif /* USE_BITFORCE || USE_BFLSC */
 	enum dev_enable deven;
 	int accepted;
 	int rejected;
@@ -525,6 +545,9 @@ struct cgpu_info {
 
 	pthread_rwlock_t qlock;
 	struct work *queued_work;
+	unsigned int queued_count;
+
+	bool shutdown;
 };
 
 extern bool add_cgpu(struct cgpu_info*);
@@ -557,13 +580,6 @@ struct thr_info {
 	bool	work_restart;
 };
 
-extern int thr_info_create(struct thr_info *thr, pthread_attr_t *attr, void *(*start) (void *), void *arg);
-extern void thr_info_cancel(struct thr_info *thr);
-extern void thr_info_freeze(struct thr_info *thr);
-extern void nmsleep(unsigned int msecs);
-extern double us_tdiff(struct timeval *end, struct timeval *start);
-extern double tdiff(struct timeval *end, struct timeval *start);
-
 struct string_elist {
 	char *string;
 	bool free_me;
@@ -634,6 +650,16 @@ static inline void flip32(void *dest_p, const void *src_p)
 		dest[i] = swab32(src[i]);
 }
 
+static inline void flip64(void *dest_p, const void *src_p)
+{
+	uint32_t *dest = dest_p;
+	const uint32_t *src = src_p;
+	int i;
+
+	for (i = 0; i < 16; i++)
+		dest[i] = swab32(src[i]);
+}
+
 static inline void flip80(void *dest_p, const void *src_p)
 {
 	uint32_t *dest = dest_p;
@@ -644,6 +670,39 @@ static inline void flip80(void *dest_p, const void *src_p)
 		dest[i] = swab32(src[i]);
 }
 
+static inline void flip128(void *dest_p, const void *src_p)
+{
+	uint32_t *dest = dest_p;
+	const uint32_t *src = src_p;
+	int i;
+
+	for (i = 0; i < 32; i++)
+		dest[i] = swab32(src[i]);
+}
+
+/* For flipping to the correct endianness if necessary */
+#if defined(__BIG_ENDIAN__) || defined(MIPSEB)
+static inline void endian_flip32(void *dest_p, const void *src_p)
+{
+	flip32(dest_p, src_p);
+}
+
+static inline void endian_flip128(void *dest_p, const void *src_p)
+{
+	flip128(dest_p, src_p);
+}
+#else
+static inline void
+endian_flip32(void __maybe_unused *dest_p, const void __maybe_unused *src_p)
+{
+}
+
+static inline void
+endian_flip128(void __maybe_unused *dest_p, const void __maybe_unused *src_p)
+{
+}
+#endif
+
 extern void quit(int status, const char *format, ...);
 
 static inline void mutex_lock(pthread_mutex_t *lock)
@@ -784,14 +843,19 @@ extern bool opt_restart;
 extern char *opt_icarus_options;
 extern char *opt_icarus_timing;
 extern bool opt_worktime;
+#ifdef USE_AVALON
+extern char *opt_avalon_options;
+#endif
 #ifdef USE_USBUTILS
 extern char *opt_usb_select;
 extern int opt_usbdump;
 extern bool opt_usb_list_all;
+extern sem_t usb_resource_sem;
 #endif
 #ifdef USE_BITFORCE
 extern bool opt_bfl_noncerange;
 #endif
+extern bool ping;
 extern int swork_id;
 
 extern pthread_rwlock_t netacc_lock;
@@ -821,6 +885,7 @@ extern int opt_expiry;
 
 #ifdef USE_USBUTILS
 extern pthread_mutex_t cgusb_lock;
+extern pthread_mutex_t cgusbres_lock;
 #endif
 
 extern cglock_t control_lock;
@@ -835,6 +900,7 @@ extern pthread_cond_t restart_cond;
 
 extern void thread_reportin(struct thr_info *thr);
 extern void clear_stratum_shares(struct pool *pool);
+extern void set_target(unsigned char *dest_target, double diff);
 extern int restart_wait(unsigned int mstime);
 
 extern void kill_work(void);
@@ -874,7 +940,6 @@ extern bool hotplug_mode;
 extern int hotplug_time;
 extern struct list_head scan_devices;
 extern int nDevs;
-extern int opt_n_threads;
 extern int num_processors;
 extern int hw_errors;
 extern bool use_syslog;
@@ -890,13 +955,10 @@ extern bool opt_scrypt;
 #endif
 extern double total_secs;
 extern int mining_threads;
-extern struct cgpu_info *cpus;
 extern int total_devices;
 extern struct cgpu_info **devices;
 extern int total_pools;
 extern struct pool **pools;
-extern const char *algo_names[];
-extern enum sha256_algos opt_algo;
 extern struct strategies strategies[];
 extern enum pool_strategy pool_strategy;
 extern int opt_rotate_period;
@@ -1054,7 +1116,7 @@ struct pool {
 	/* Stratum variables */
 	char *stratum_url;
 	char *stratum_port;
-	CURL *stratum_curl;
+	struct addrinfo stratum_hints;
 	SOCKETTYPE sock;
 	char *sockbuf;
 	size_t sockbuf_size;
@@ -1069,8 +1131,10 @@ struct pool {
 	bool stratum_init;
 	bool stratum_notify;
 	struct stratum_work swork;
-	pthread_t stratum_thread;
+	pthread_t stratum_sthread;
+	pthread_t stratum_rthread;
 	pthread_mutex_t stratum_lock;
+	struct thread_q *stratum_q;
 	int sshares; /* stratum shares submitted waiting on response */
 
 	/* GBT  variables */
@@ -1105,7 +1169,11 @@ struct work {
 	unsigned char	target[32];
 	unsigned char	hash[32];
 
-	double	device_diff;
+#ifdef USE_SCRYPT
+	unsigned char	device_target[32];
+#endif
+	double		device_diff;
+	uint64_t	share_diff;
 
 	int		rolls;
 
@@ -1143,6 +1211,11 @@ struct work {
 
 	double		work_difficulty;
 
+	// Allow devices to identify work if multiple sub-devices
+	int		subid;
+	// Allow devices to flag work for their own purposes
+	bool		devflag;
+
 	struct timeval	tv_getwork;
 	struct timeval	tv_getwork_reply;
 	struct timeval	tv_cloned;
@@ -1180,8 +1253,11 @@ struct modminer_fpga_state {
 #endif
 
 extern void get_datestamp(char *, struct timeval *);
-extern void submit_nonce(struct thr_info *thr, struct work *work, uint32_t nonce);
+extern void inc_hw_errors(struct thr_info *thr);
+extern bool submit_nonce(struct thr_info *thr, struct work *work, uint32_t nonce);
 extern struct work *get_queued(struct cgpu_info *cgpu);
+extern struct work *__find_work_bymidstate(struct work *que, char *midstate, size_t midstatelen, char *data, int offset, size_t datalen);
+extern struct work *find_queued_work_bymidstate(struct cgpu_info *cgpu, char *midstate, size_t midstatelen, char *data, int offset, size_t datalen);
 extern void work_completed(struct cgpu_info *cgpu, struct work *work);
 extern void hash_queued_work(struct thr_info *mythr);
 extern void tailsprintf(char *f, const char *fmt, ...);
@@ -1198,6 +1274,7 @@ extern void zero_stats(void);
 extern void default_save_file(char *filename);
 extern bool log_curses_only(int prio, const char *f, va_list ap);
 extern void clear_logwin(void);
+extern void logwin_update(void);
 extern bool pool_tclear(struct pool *pool, bool *var);
 extern struct thread_q *tq_new(void);
 extern void tq_free(struct thread_q *tq);
