@@ -22,6 +22,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #ifndef WIN32
+#include <fcntl.h>
 # ifdef __linux
 #  include <sys/prctl.h>
 # endif
@@ -30,8 +31,10 @@
 # include <netinet/tcp.h>
 # include <netdb.h>
 #else
+# include <windows.h>
 # include <winsock2.h>
 # include <ws2tcpip.h>
+# include <mmsystem.h>
 #endif
 
 #include "miner.h"
@@ -197,6 +200,38 @@ out:
 	return ptrlen;
 }
 
+static void keep_sockalive(SOCKETTYPE fd)
+{
+	const int tcp_one = 1;
+#ifndef WIN32
+	const int tcp_keepidle = 45;
+	const int tcp_keepintvl = 30;
+	int flags = fcntl(fd, F_GETFL, 0);
+
+	fcntl(fd, F_SETFL, O_NONBLOCK | flags);
+#else
+	u_long flags = 1;
+
+	ioctlsocket(fd, FIONBIO, &flags);
+#endif
+
+	setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (const void *)&tcp_one, sizeof(tcp_one));
+	if (!opt_delaynet)
+#ifndef __linux
+		setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (const void *)&tcp_one, sizeof(tcp_one));
+#else /* __linux */
+		setsockopt(fd, SOL_TCP, TCP_NODELAY, (const void *)&tcp_one, sizeof(tcp_one));
+	setsockopt(fd, SOL_TCP, TCP_KEEPCNT, &tcp_one, sizeof(tcp_one));
+	setsockopt(fd, SOL_TCP, TCP_KEEPIDLE, &tcp_keepidle, sizeof(tcp_keepidle));
+	setsockopt(fd, SOL_TCP, TCP_KEEPINTVL, &tcp_keepintvl, sizeof(tcp_keepintvl));
+#endif /* __linux */
+
+#ifdef __APPLE_CC__
+	setsockopt(fd, IPPROTO_TCP, TCP_KEEPALIVE, &tcp_keepintvl, sizeof(tcp_keepintvl));
+#endif /* __APPLE_CC__ */
+
+}
+
 #if CURL_HAS_KEEPALIVE
 static void keep_curlalive(CURL *curl)
 {
@@ -208,30 +243,7 @@ static void keep_curlalive(CURL *curl)
 	curl_easy_setopt(curl, CURLOPT_TCP_KEEPIDLE, tcp_keepidle);
 	curl_easy_setopt(curl, CURLOPT_TCP_KEEPINTVL, tcp_keepintvl);
 }
-
-static void keep_alive(CURL *curl, __maybe_unused SOCKETTYPE fd)
-{
-	keep_curlalive(curl);
-}
 #else
-static void keep_sockalive(SOCKETTYPE fd)
-{
-	const int tcp_keepidle = 45;
-	const int tcp_keepintvl = 30;
-	const int keepalive = 1;
-	const int tcp_keepcnt = 1;
-
-	setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, &keepalive, sizeof(keepalive));
-# ifdef __linux
-	setsockopt(fd, SOL_TCP, TCP_KEEPCNT, &tcp_keepcnt, sizeof(tcp_keepcnt));
-	setsockopt(fd, SOL_TCP, TCP_KEEPIDLE, &tcp_keepidle, sizeof(tcp_keepidle));
-	setsockopt(fd, SOL_TCP, TCP_KEEPINTVL, &tcp_keepintvl, sizeof(tcp_keepintvl));
-# endif /* __linux */
-# ifdef __APPLE_CC__
-	setsockopt(fd, IPPROTO_TCP, TCP_KEEPALIVE, &tcp_keepintvl, sizeof(tcp_keepintvl));
-# endif /* __APPLE_CC__ */
-}
-
 static void keep_curlalive(CURL *curl)
 {
 	SOCKETTYPE sock;
@@ -239,11 +251,6 @@ static void keep_curlalive(CURL *curl)
 	curl_easy_getinfo(curl, CURLINFO_LASTSOCKET, (long *)&sock);
 	keep_sockalive(sock);
 }
-
-static void keep_alive(CURL __maybe_unused *curl, SOCKETTYPE fd)
-{
-	keep_sockalive(fd);
-}
 #endif
 
 static void last_nettime(struct timeval *last)
@@ -257,7 +264,7 @@ static void last_nettime(struct timeval *last)
 static void set_nettime(void)
 {
 	wr_lock(&netacc_lock);
-	gettimeofday(&nettime, NULL);
+	cgtime(&nettime);
 	wr_unlock(&netacc_lock);
 }
 
@@ -381,7 +388,7 @@ json_t *json_rpc_call(CURL *curl, const char *url,
 			long long now_msecs, last_msecs;
 			struct timeval now, last;
 
-			gettimeofday(&now, NULL);
+			cgtime(&now);
 			last_nettime(&last);
 			now_msecs = (long long)now.tv_sec * 1000;
 			now_msecs += now.tv_usec / 1000;
@@ -451,7 +458,7 @@ json_t *json_rpc_call(CURL *curl, const char *url,
 		applog(LOG_INFO, "JSON decode failed(%d): %s", err.line, err.text);
 
 		if (opt_protocol)
-			applog(LOG_DEBUG, "JSON protocol response:\n%s", all_data.buf);
+			applog(LOG_DEBUG, "JSON protocol response:\n%s", (char *)(all_data.buf));
 
 		goto err_out;
 	}
@@ -637,13 +644,9 @@ bool fulltest(const unsigned char *hash, const unsigned char *target)
 	swap256(target_swap, target);
 
 	for (i = 0; i < 32/4; i++) {
-#ifdef MIPSEB
-		uint32_t h32tmp = hash32[i];
-		uint32_t t32tmp = swab32(target32[i]);
-#else
-		uint32_t h32tmp = swab32(hash32[i]);
-		uint32_t t32tmp = target32[i];
-#endif
+		uint32_t h32tmp = htobe32(hash32[i]);
+		uint32_t t32tmp = htole32(target32[i]);
+
 		target32[i] = swab32(target32[i]);	/* for printing */
 
 		if (h32tmp > t32tmp) {
@@ -663,7 +666,7 @@ bool fulltest(const unsigned char *hash, const unsigned char *target)
 		applog(LOG_DEBUG, " Proof: %s\nTarget: %s\nTrgVal? %s",
 			hash_str,
 			target_str,
-			rc ? "YES (hash < target)" :
+			rc ? "YES (hash <= target)" :
 			     "no (false positive; hash > target)");
 
 		free(hash_str);
@@ -710,9 +713,7 @@ void tq_free(struct thread_q *tq)
 static void tq_freezethaw(struct thread_q *tq, bool frozen)
 {
 	mutex_lock(&tq->mutex);
-
 	tq->frozen = frozen;
-
 	pthread_cond_signal(&tq->cond);
 	mutex_unlock(&tq->mutex);
 }
@@ -740,14 +741,12 @@ bool tq_push(struct thread_q *tq, void *data)
 	INIT_LIST_HEAD(&ent->q_node);
 
 	mutex_lock(&tq->mutex);
-
 	if (!tq->frozen) {
 		list_add_tail(&ent->q_node, &tq->q);
 	} else {
 		free(ent);
 		rc = false;
 	}
-
 	pthread_cond_signal(&tq->cond);
 	mutex_unlock(&tq->mutex);
 
@@ -761,7 +760,6 @@ void *tq_pop(struct thread_q *tq, const struct timespec *abstime)
 	int rc;
 
 	mutex_lock(&tq->mutex);
-
 	if (!list_empty(&tq->q))
 		goto pop;
 
@@ -773,16 +771,15 @@ void *tq_pop(struct thread_q *tq, const struct timespec *abstime)
 		goto out;
 	if (list_empty(&tq->q))
 		goto out;
-
 pop:
 	ent = list_entry(tq->q.next, struct tq_ent, q_node);
 	rval = ent->data;
 
 	list_del(&ent->q_node);
 	free(ent);
-
 out:
 	mutex_unlock(&tq->mutex);
+
 	return rval;
 }
 
@@ -831,6 +828,9 @@ void nmsleep(unsigned int msecs)
 	int ret;
 	ldiv_t d;
 
+#ifdef WIN32
+	timeBeginPeriod(1);
+#endif
 	d = ldiv(msecs, 1000);
 	tleft.tv_sec = d.quot;
 	tleft.tv_nsec = d.rem * 1000000;
@@ -839,6 +839,71 @@ void nmsleep(unsigned int msecs)
 		twait.tv_nsec = tleft.tv_nsec;
 		ret = nanosleep(&twait, &tleft);
 	} while (ret == -1 && errno == EINTR);
+#ifdef WIN32
+	timeEndPeriod(1);
+#endif
+}
+
+/* Same for usecs */
+void nusleep(unsigned int usecs)
+{
+	struct timespec twait, tleft;
+	int ret;
+	ldiv_t d;
+
+#ifdef WIN32
+	timeBeginPeriod(1);
+#endif
+	d = ldiv(usecs, 1000000);
+	tleft.tv_sec = d.quot;
+	tleft.tv_nsec = d.rem * 1000;
+	do {
+		twait.tv_sec = tleft.tv_sec;
+		twait.tv_nsec = tleft.tv_nsec;
+		ret = nanosleep(&twait, &tleft);
+	} while (ret == -1 && errno == EINTR);
+#ifdef WIN32
+	timeEndPeriod(1);
+#endif
+}
+
+/* This is a cgminer gettimeofday wrapper. Since we always call gettimeofday
+ * with tz set to NULL, and windows' default resolution is only 15ms, this
+ * gives us higher resolution times on windows. */
+void cgtime(struct timeval *tv)
+{
+#ifdef WIN32
+	timeBeginPeriod(1);
+#endif
+	gettimeofday(tv, NULL);
+#ifdef WIN32
+	timeEndPeriod(1);
+#endif
+}
+
+void subtime(struct timeval *a, struct timeval *b)
+{
+	timersub(a, b, b);
+}
+
+void addtime(struct timeval *a, struct timeval *b)
+{
+	timeradd(a, b, b);
+}
+
+bool time_more(struct timeval *a, struct timeval *b)
+{
+	return timercmp(a, b, >);
+}
+
+bool time_less(struct timeval *a, struct timeval *b)
+{
+	return timercmp(a, b, <);
+}
+
+void copy_time(struct timeval *dest, const struct timeval *src)
+{
+	memcpy(dest, src, sizeof(struct timeval));
 }
 
 /* Returns the microseconds difference between end and start times as a double */
@@ -898,36 +963,42 @@ bool extract_sockaddr(struct pool *pool, char *url)
 	return true;
 }
 
+enum send_ret {
+	SEND_OK,
+	SEND_SELECTFAIL,
+	SEND_SENDFAIL,
+	SEND_INACTIVE
+};
+
 /* Send a single command across a socket, appending \n to it. This should all
  * be done under stratum lock except when first establishing the socket */
-static bool __stratum_send(struct pool *pool, char *s, ssize_t len)
+static enum send_ret __stratum_send(struct pool *pool, char *s, ssize_t len)
 {
 	SOCKETTYPE sock = pool->sock;
 	ssize_t ssent = 0;
 
-	if (opt_protocol)
-		applog(LOG_DEBUG, "SEND: %s", s);
-
 	strcat(s, "\n");
 	len++;
 
 	while (len > 0 ) {
-		struct timeval timeout = {0, 0};
+		struct timeval timeout = {1, 0};
 		ssize_t sent;
 		fd_set wd;
 
 		FD_ZERO(&wd);
 		FD_SET(sock, &wd);
-		if (select(sock + 1, NULL, &wd, NULL, &timeout) < 1) {
-			applog(LOG_DEBUG, "Write select failed on pool %d sock", pool->pool_no);
-			return false;
-		}
+		if (select(sock + 1, NULL, &wd, NULL, &timeout) < 1)
+			return SEND_SELECTFAIL;
+#ifdef __APPLE__
+		sent = send(pool->sock, s + ssent, len, SO_NOSIGPIPE);
+#elif WIN32
 		sent = send(pool->sock, s + ssent, len, 0);
+#else
+		sent = send(pool->sock, s + ssent, len, MSG_NOSIGNAL);
+#endif
 		if (sent < 0) {
-			if (errno != EAGAIN && errno != EWOULDBLOCK) {
-				applog(LOG_DEBUG, "Failed to curl_easy_send in stratum_send");
-				return false;
-			}
+			if (!sock_blocks())
+				return SEND_SENDFAIL;
 			sent = 0;
 		}
 		ssent += sent;
@@ -937,21 +1008,37 @@ static bool __stratum_send(struct pool *pool, char *s, ssize_t len)
 	pool->cgminer_pool_stats.times_sent++;
 	pool->cgminer_pool_stats.bytes_sent += ssent;
 	pool->cgminer_pool_stats.net_bytes_sent += ssent;
-	return true;
+	return SEND_OK;
 }
 
 bool stratum_send(struct pool *pool, char *s, ssize_t len)
 {
-	bool ret = false;
+	enum send_ret ret = SEND_INACTIVE;
+
+	if (opt_protocol)
+		applog(LOG_DEBUG, "SEND: %s", s);
 
 	mutex_lock(&pool->stratum_lock);
 	if (pool->stratum_active)
 		ret = __stratum_send(pool, s, len);
-	else
-		applog(LOG_DEBUG, "Stratum send failed due to no pool stratum_active");
 	mutex_unlock(&pool->stratum_lock);
 
-	return ret;
+	/* This is to avoid doing applog under stratum_lock */
+	switch (ret) {
+		default:
+		case SEND_OK:
+			break;
+		case SEND_SELECTFAIL:
+			applog(LOG_DEBUG, "Write select failed on pool %d sock", pool->pool_no);
+			break;
+		case SEND_SENDFAIL:
+			applog(LOG_DEBUG, "Failed to curl_easy_send in stratum_send");
+			break;
+		case SEND_INACTIVE:
+			applog(LOG_DEBUG, "Stratum send failed due to no pool stratum_active");
+			break;
+	}
+	return (ret == SEND_OK);
 }
 
 static bool socket_full(struct pool *pool, bool wait)
@@ -966,7 +1053,7 @@ static bool socket_full(struct pool *pool, bool wait)
 	if (wait)
 		timeout.tv_sec = 60;
 	else
-		timeout.tv_sec = 0;
+		timeout.tv_sec = 1;
 	if (select(sock + 1, &rd, NULL, NULL, &timeout) > 0)
 		return true;
 	return false;
@@ -1011,7 +1098,8 @@ static void recalloc_sock(struct pool *pool, size_t len)
 	if (new < pool->sockbuf_size)
 		return;
 	new = new + (RBUFSIZE - (new % RBUFSIZE));
-	applog(LOG_DEBUG, "Recallocing pool sockbuf to %d", new);
+	// Avoid potentially recursive locking
+	// applog(LOG_DEBUG, "Recallocing pool sockbuf to %d", new);
 	pool->sockbuf = realloc(pool->sockbuf, new);
 	if (!pool->sockbuf)
 		quit(1, "Failed to realloc pool sockbuf in recalloc_sock");
@@ -1029,29 +1117,35 @@ char *recv_line(struct pool *pool)
 	if (!strstr(pool->sockbuf, "\n")) {
 		struct timeval rstart, now;
 
-		gettimeofday(&rstart, NULL);
+		cgtime(&rstart);
 		if (!socket_full(pool, true)) {
 			applog(LOG_DEBUG, "Timed out waiting for data on socket_full");
 			goto out;
 		}
 
-		mutex_lock(&pool->stratum_lock);
 		do {
 			char s[RBUFSIZE];
-			size_t slen, n;
+			size_t slen;
+			ssize_t n;
 
 			memset(s, 0, RBUFSIZE);
 			n = recv(pool->sock, s, RECVSIZE, 0);
-			if (n < 1 && errno != EAGAIN && errno != EWOULDBLOCK) {
-				applog(LOG_DEBUG, "Failed to recv sock in recv_line");
+			if (!n) {
+				applog(LOG_DEBUG, "Socket closed waiting in recv_line");
 				break;
 			}
-			slen = strlen(s);
-			recalloc_sock(pool, slen);
-			strcat(pool->sockbuf, s);
-			gettimeofday(&now, NULL);
+			if (n < 0) {
+				if (!sock_blocks() || !socket_full(pool, false)) {
+					applog(LOG_DEBUG, "Failed to recv sock in recv_line");
+					break;
+				}
+			} else {
+				slen = strlen(s);
+				recalloc_sock(pool, slen);
+				strcat(pool->sockbuf, s);
+			}
+			cgtime(&now);
 		} while (tdiff(&now, &rstart) < 60 && !strstr(pool->sockbuf, "\n"));
-		mutex_unlock(&pool->stratum_lock);
 	}
 
 	buflen = strlen(pool->sockbuf);
@@ -1216,17 +1310,21 @@ out:
 
 static bool parse_diff(struct pool *pool, json_t *val)
 {
-	double diff;
+	double old_diff, diff;
 
 	diff = json_number_value(json_array_get(val, 0));
 	if (diff == 0)
 		return false;
 
 	cg_wlock(&pool->data_lock);
+	old_diff = pool->swork.diff;
 	pool->swork.diff = diff;
 	cg_wunlock(&pool->data_lock);
 
-	applog(LOG_DEBUG, "Pool %d difficulty set to %f", pool->pool_no, diff);
+	if (old_diff != diff)
+		applog(LOG_NOTICE, "Pool %d difficulty changed to %f", pool->pool_no, diff);
+	else
+		applog(LOG_DEBUG, "Pool %d difficulty set to %f", pool->pool_no, diff);
 
 	return true;
 }
@@ -1415,22 +1513,56 @@ out:
 	return ret;
 }
 
-static bool setup_stratum_curl(struct pool *pool)
+static bool setup_stratum_socket(struct pool *pool)
 {
-	char curl_err_str[CURL_ERROR_SIZE];
-	CURL *curl = NULL;
-	double byte_count;
-	char s[RBUFSIZE];
+	struct addrinfo *servinfo, *hints, *p;
+	int sockd;
 
 	mutex_lock(&pool->stratum_lock);
 	pool->stratum_active = false;
-	if (pool->stratum_curl)
-		curl_easy_cleanup(pool->stratum_curl);
-	pool->stratum_curl = curl_easy_init();
-	if (unlikely(!pool->stratum_curl))
-		quit(1, "Failed to curl_easy_init in initiate_stratum");
+	if (pool->sock)
+		CLOSESOCKET(pool->sock);
+	pool->sock = 0;
 	mutex_unlock(&pool->stratum_lock);
-	curl = pool->stratum_curl;
+
+	hints = &pool->stratum_hints;
+	memset(hints, 0, sizeof(struct addrinfo));
+	hints->ai_family = AF_UNSPEC;
+	hints->ai_socktype = SOCK_STREAM;
+	if (getaddrinfo(pool->sockaddr_url, pool->stratum_port, hints, &servinfo) != 0) {
+		if (!pool->probed) {
+			applog(LOG_WARNING, "Failed to resolve (?wrong URL) %s:%s",
+			       pool->sockaddr_url, pool->stratum_port);
+			pool->probed = true;
+		} else {
+			applog(LOG_INFO, "Failed to getaddrinfo for %s:%s",
+			       pool->sockaddr_url, pool->stratum_port);
+		}
+		return false;
+	}
+
+	for (p = servinfo; p != NULL; p = p->ai_next) {
+		sockd = socket(p->ai_family, p->ai_socktype, p->ai_protocol);
+		if (sockd == -1) {
+			applog(LOG_DEBUG, "Failed socket");
+			continue;
+		}
+
+		if (connect(sockd, p->ai_addr, p->ai_addrlen) == -1) {
+			CLOSESOCKET(sockd);
+			applog(LOG_DEBUG, "Failed connect");
+			continue;
+		}
+
+		break;
+	}
+	if (p == NULL) {
+		applog(LOG_INFO, "Failed to find a stratum servinfo on %s:%s",
+		       pool->sockaddr_url, pool->stratum_port);
+		freeaddrinfo(servinfo);
+		return false;
+	}
+	freeaddrinfo(servinfo);
 
 	if (!pool->sockbuf) {
 		pool->sockbuf = calloc(RBUFSIZE, 1);
@@ -1439,38 +1571,8 @@ static bool setup_stratum_curl(struct pool *pool)
 		pool->sockbuf_size = RBUFSIZE;
 	}
 
-	/* Create a http url for use with curl */
-	sprintf(s, "http://%s:%s", pool->sockaddr_url, pool->stratum_port);
-
-	curl_easy_setopt(curl, CURLOPT_FRESH_CONNECT, 1);
-	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 30);
-	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, curl_err_str);
-	curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
-	curl_easy_setopt(curl, CURLOPT_URL, s);
-	curl_easy_setopt(curl, CURLOPT_TCP_NODELAY, 1);
-	curl_easy_setopt(curl, CURLOPT_USE_SSL, CURLUSESSL_TRY);
-	if (pool->rpc_proxy) {
-		curl_easy_setopt(curl, CURLOPT_PROXY, pool->rpc_proxy);
-		curl_easy_setopt(curl, CURLOPT_PROXYTYPE, pool->rpc_proxytype);
-	} else if (opt_socks_proxy) {
-		curl_easy_setopt(curl, CURLOPT_PROXY, opt_socks_proxy);
-		curl_easy_setopt(curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS4);
-	}
-	curl_easy_setopt(curl, CURLOPT_CONNECT_ONLY, 1);
-	if (curl_easy_perform(curl)) {
-		applog(LOG_INFO, "Stratum connect failed to pool %d: %s", pool->pool_no, curl_err_str);
-		return false;
-	}
-	curl_easy_getinfo(curl, CURLINFO_LASTSOCKET, (long *)&pool->sock);
-	keep_alive(curl, pool->sock);
-
-	pool->cgminer_pool_stats.times_sent++;
-	if (curl_easy_getinfo(curl, CURLINFO_SIZE_UPLOAD, &byte_count) == CURLE_OK)
-		pool->cgminer_pool_stats.bytes_sent += byte_count;
-	pool->cgminer_pool_stats.times_received++;
-	if (curl_easy_getinfo(curl, CURLINFO_SIZE_DOWNLOAD, &byte_count) == CURLE_OK)
-		pool->cgminer_pool_stats.bytes_received += byte_count;
-
+	pool->sock = sockd;
+	keep_sockalive(sockd);
 	return true;
 }
 
@@ -1506,20 +1608,12 @@ void suspend_stratum(struct pool *pool)
 {
 	clear_sockbuf(pool);
 	applog(LOG_INFO, "Closing socket for stratum pool %d", pool->pool_no);
+
 	mutex_lock(&pool->stratum_lock);
 	pool->stratum_active = pool->stratum_notify = false;
-	if (pool->stratum_curl) {
-#if CURL_HAS_KEEPALIVE
-		curl_easy_cleanup(pool->stratum_curl);
-#else
-		/* Old versions of libcurl seem to crash occasionally on this since
-		* the socket is modified in keep_sockalive in ways curl does not
-		* know about so sacrifice the ram knowing we leak one curl handle
-		* every time we disconnect stratum. */
+	if (pool->sock)
 		CLOSESOCKET(pool->sock);
-#endif
-	}
-	pool->stratum_curl = NULL;
+	pool->sock = 0;
 	mutex_unlock(&pool->stratum_lock);
 }
 
@@ -1532,7 +1626,7 @@ bool initiate_stratum(struct pool *pool)
 	int n2size;
 
 resend:
-	if (!setup_stratum_curl(pool)) {
+	if (!setup_stratum_socket(pool)) {
 		sockd = false;
 		goto out;
 	}
@@ -1550,7 +1644,7 @@ resend:
 			sprintf(s, "{\"id\": %d, \"method\": \"mining.subscribe\", \"params\": [\""PACKAGE"/"VERSION"\"]}", swork_id++);
 	}
 
-	if (!__stratum_send(pool, s, strlen(s))) {
+	if (__stratum_send(pool, s, strlen(s)) != SEND_OK) {
 		applog(LOG_DEBUG, "Failed to send s in initiate_stratum");
 		goto out;
 	}
@@ -1643,6 +1737,7 @@ out:
 			free(pool->nonce1);
 			pool->sessionid = pool->nonce1 = NULL;
 			cg_wunlock(&pool->data_lock);
+
 			applog(LOG_DEBUG, "Failed to resume stratum, trying afresh");
 			noresume = true;
 			goto resend;
@@ -1726,6 +1821,40 @@ void *realloc_strcat(char *ptr, char *s)
 	return ret;
 }
 
+/* Make a text readable version of a string using 0xNN for < ' ' or > '~'
+ * Including 0x00 at the end
+ * You must free the result yourself */
+void *str_text(char *ptr)
+{
+	unsigned char *uptr;
+	char *ret, *txt;
+
+	if (ptr == NULL) {
+		ret = strdup("(null)");
+
+		if (unlikely(!ret))
+			quit(1, "Failed to malloc in text_str null");
+	}
+
+	uptr = (unsigned char *)ptr;
+
+	ret = txt = malloc(strlen(ptr)*4+5); // Guaranteed >= needed
+	if (unlikely(!txt))
+		quit(1, "Failed to malloc in text_str txt");
+
+	do {
+		if (*uptr < ' ' || *uptr > '~') {
+			sprintf(txt, "0x%02x", *uptr);
+			txt += 4;
+		} else
+			*(txt++) = *uptr;
+	} while (*(uptr++));
+
+	*txt = '\0';
+
+	return ret;
+}
+
 void RenameThread(const char* name)
 {
 #if defined(PR_SET_NAME)
