@@ -14,6 +14,10 @@
 	#define CLOSESOCKET close
 
 	#define SOCKERRMSG strerror(errno)
+	static inline bool sock_blocks(void)
+	{
+		return (errno == EAGAIN || errno == EWOULDBLOCK);
+	}
 #elif defined WIN32
 	#include <ws2tcpip.h>
 	#include <winsock2.h>
@@ -27,6 +31,10 @@
 	extern char *WSAErrorMsg(void);
 	#define SOCKERRMSG WSAErrorMsg()
 
+	static inline bool sock_blocks(void)
+	{
+		return (WSAGetLastError() == WSAEWOULDBLOCK);
+	}
 	#ifndef SHUT_RDWR
 	#define SHUT_RDWR SD_BOTH
 	#endif
@@ -42,9 +50,23 @@
 #define JSON_LOADS(str, err_ptr) json_loads((str), (err_ptr))
 #endif
 
+struct thr_info;
 struct pool;
 enum dev_reason;
 struct cgpu_info;
+int thr_info_create(struct thr_info *thr, pthread_attr_t *attr, void *(*start) (void *), void *arg);
+void thr_info_freeze(struct thr_info *thr);
+void thr_info_cancel(struct thr_info *thr);
+void nmsleep(unsigned int msecs);
+void nusleep(unsigned int usecs);
+void cgtime(struct timeval *tv);
+void subtime(struct timeval *a, struct timeval *b);
+void addtime(struct timeval *a, struct timeval *b);
+bool time_more(struct timeval *a, struct timeval *b);
+bool time_less(struct timeval *a, struct timeval *b);
+void copy_time(struct timeval *dest, const struct timeval *src);
+double us_tdiff(struct timeval *end, struct timeval *start);
+double tdiff(struct timeval *end, struct timeval *start);
 bool stratum_send(struct pool *pool, char *s, ssize_t len);
 bool sock_full(struct pool *pool);
 char *recv_line(struct pool *pool);
@@ -56,6 +78,7 @@ bool restart_stratum(struct pool *pool);
 void suspend_stratum(struct pool *pool);
 void dev_error(struct cgpu_info *dev, enum dev_reason reason);
 void *realloc_strcat(char *ptr, char *s);
+void *str_text(char *ptr);
 void RenameThread(const char* name);
 
 /* Align a size_t to 4 byte boundaries for fussy arches */
